/***************************************************************************
 *   Nipper - The network infrastructure parser                            *
 *   Copyright Titania Limited 2009                                        *
 *   http://www.titania.co.uk                                              *
 *                                                                         *
 *   The software license should have been provided with this product in   *
 *   a file named LICENSE. All unlicensed use of Nipper is strictly        *
 *   prohibited.                                                           *
 *                                                                         *
 *   The license can be obtained online from the web site.                 *
 *                                                                         *
 ***************************************************************************/

#ifndef libnipper_H
#define libnipper_H

extern "C"
{

#include "config.h"

class Device;
class Report;


/**
 * @short The main Nipper class.
 *
 * The Nipper class provides the main processing method for the processing
 * of device configurations. Use the class Config to configure Nipper prior to
 * processing and reporting.
 *
 * <b>Example:</b>\n
 *
 * \code
 *   #include <libnipper/libnipper.h>
 *   #include <libnipper/config.h>
 *   ...
 *   Nipper *nipper = 0;
 *   int errorCode = 0;
 *   ...
 *   // Create Nipper
 *   nipper = new (Nipper);
 *
 *   // Set the device configuration file for nipper to process...
 *   nipper->config->inputSource = "firewall.config";
 *
 *   // Process the configuration file...
 *   errorCode = nipper->process();
 *   if (errorCode == 0)
 *   {
 *
 *       // Generate the report from the configuration...
 *       errorCode = nipper->generateReport();
 *       if (errorCode == 0)
 *       {
 *
 *           // Output the report...
 *           errorCode = writeReport();
 *       }
 *   }
 *
 *   // If an error occured, output the error text...
 *   if (errorCode > 0)
 *       printf("Error: %s\n", nipper->getErrorText(errorCode));
 * \endcode
 */
class Nipper
{

  public:

	/**
	 * \brief Get the Nipper library version
	 *
	 * This method returns the Nipper library version.
	 *
	 * \return            The Nipper library version text.
	 */
	const char *getVersion();

	/**
	 * \brief The Nipper configuration settings
	 *
	 * Set the properties of this to configure Nipper.
	 */
	Config *config;

	/**
	 * \brief Process the device configuration file
	 *
	 * This method tells Nipper to process the configuration file based
	 * on the settings. Once the configuration has been processed a report
	 * can be generated.
	 *
	 * \return      It returns 0 on success, otherwise it indicates an
	 *              error of some kind. The error text can be retreived
	 *              using the getErrorText method.
     * \see Nipper::getErrorText
	 */
	int process();

	/**
	 * \brief Generate the report
	 *
	 * This method tells Nipper to generate a report based on the results of
	 * the configuration processing and the Nipper settings. It does not output
	 * the report.
	 * The report generation settings can be modified and generateReport run again
	 * without having to process the devices configuration again. This could be
	 * useful if you want to create seperate reports for configuration, security
	 * or any other report type.
	 *
	 * \return      It returns 0 on success, otherwise it indicates an
	 *              error of some kind. The error text can be retreived
	 *              using the getErrorText method.
     * \see Nipper::getErrorText
	 * \see Nipper::process
	 * \see Nipper::writeReport
	 */
	int generateReport();

	/**
	 * \brief Write the report to a file
	 *
	 * This method tells Nipper to write out a generated report. This method does not
	 * generate the report, the generateReport method should be called first.
	 *
	 * The report output settings can be modified and writeReport run again. This
	 * can enable the output to multiple report formats, without having to repeat
	 * any other processing.
	 *
	 * \return      It returns 0 on success, otherwise it indicates an
	 *              error of some kind. The error text can be retreived
	 *              using the getErrorText method.
     * \see Nipper::getErrorText
	 * \see Nipper::process
	 * \see Nipper::generateReport
	 */
	int writeReport();

	/**
	 * \brief Get the error text
	 *
	 * This method gets the error text based on the code provided.
	 *
	 * \param errorCode   The error code returned by the process method.
	 * \return            The error text for the supplied code is returned.
	 * \see Nipper::process
	 */
	const char *getErrorText(int errorCode);

	/**
	 * \brief The Supported Nipper Device Configuration Type
	 *
	 * This enumeration is used to indicate what form the configuration
	 * is expected to take. For example, Cisco ASA configuration will be
	 * a file and a Nokia IP firewall configuration will be directory.
	 *
	 * \see DeviceTypes
	 */
	enum ConfigType
	{
		configFile = 0,
		configDir = 1,
		configEither = 2
	};

	/**
	 * \brief The Supported Nipper Device Types Structure
	 *
	 * This structure is used to create a list of devices that Nipper
	 * supports and can be used to specify what type of device the configuration
	 * is for.
	 *
	 * - deviceDescription (Contains an textual description of the device type)
	 * - commandLineOption (Contains the devices recommended cli option)
	 * - deviceId (Contains the ID that should be used to set the deviceType property in the Config class)
	 * - type (Identifies whether Nipper is expecting a configuration file or a configuration directory)
	 * - next (Points to the next entry in the list of supported devices, or 0 if it is the last entry)
	 *
	 * \see deviceTypes
	 * \see ConfigType
	 */
	struct SupportedDeviceTypes
	{
		const char *deviceDescription;
		const char *commandLineOption;
		Config::DeviceTypes deviceId;
		ConfigType type;
		struct SupportedDeviceTypes *next;
	};

	/**
	 * \brief The Supported Nipper Device Type List
	 *
	 * This is a Config::DeviceTypes list of devices supported by Nipper. A
	 * GUI/CLI can make use of this list to dynamically support whatever devices
	 * that Nipper supports.
	 *
	 * It is highly recommended that you use this structure to get a list of
	 * devices if you are not going to let Nipper do its auto-detection. Using
	 * this structure means that when new devices are added to Nipper your
	 * software will automatically support them. In the future, device support
	 * is likely to be added with the use of plugins.
	 *
	 * \see SupportedDeviceTypes
	 */
	SupportedDeviceTypes *deviceTypes;

	/**
	 * Constructor
	 */
	Nipper();

	/**
	 * Destructor
	 */
	~Nipper();

  private:
	Device *device;
	Report *report;
	bool processed;
	bool generated;
};

}

#endif
